﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.SearchModels;
using FurnitureAssemblyContracts.StoragesContracts;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyDatabaseImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FurnitureAssemblyDatabaseImplement.Implements
{
    public class ImplementerStorage : IImplementerStorage
    {
        public ImplementerViewModel? GetElement(ImplementerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ImplementerFIO) && !model.Id.HasValue && string.IsNullOrEmpty(model.Password))
            {
                return null;
            }
            using var context = new FurnitureAssemblyDatabase();
            if (model.Id.HasValue)
            {
                return context.Implementers
                    .FirstOrDefault(x => model.Id.HasValue && x.Id == model.Id)?.GetViewModel;
            }
            if (model.ImplementerFIO != null && model.Password != null)
            {
                return context.Implementers
                .FirstOrDefault(x =>
                x.ImplementerFIO.Equals(model.ImplementerFIO) && x.Password.Equals(model.Password))
                ?.GetViewModel;
            }
            return context.Implementers
                .FirstOrDefault(x =>
                x.ImplementerFIO.Equals(model.ImplementerFIO))
                ?.GetViewModel;
        }

        public List<ImplementerViewModel> GetFilteredList(ImplementerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ImplementerFIO))
            {
                return new();
            }
            using var context = new FurnitureAssemblyDatabase();
            return context.Implementers
            .Where(x => x.ImplementerFIO.Equals(model.ImplementerFIO))
           .Select(x => x.GetViewModel)
           .ToList();
        }

        public List<ImplementerViewModel> GetFullList()
        {
            using var context = new FurnitureAssemblyDatabase();
            return context.Implementers
            .Select(x => x.GetViewModel)
           .ToList();
        }

        public ImplementerViewModel? Insert(ImplementerBindingModel model)
        {
            var newImplementer = Implementer.Create(model);
            if (newImplementer == null)
            {
                return null;
            }
            using var context = new FurnitureAssemblyDatabase();
            context.Implementers.Add(newImplementer);
            context.SaveChanges();
            return newImplementer.GetViewModel;
        }

        public ImplementerViewModel? Update(ImplementerBindingModel model)
        {
            using var context = new FurnitureAssemblyDatabase();
            var client = context.Implementers.FirstOrDefault(x => x.Id == model.Id);
            if (client == null)
            {
                return null;
            }
            client.Update(model);
            context.SaveChanges();
            return client.GetViewModel;
        }

        public ImplementerViewModel? Delete(ImplementerBindingModel model)
        {
            using var context = new FurnitureAssemblyDatabase();
            var element = context.Implementers.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Implementers.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
