﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyDataModels.Enums;
using FurnitureAssemblyDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


namespace FurnitureAssemblyDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        [ForeignKey("ProductId")]
        public int FurnitureId { get; set; }
        [Required]
        public int Count { get; set; }
        [Required]
        public double Sum { get; set; }
        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;
        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now.ToUniversalTime();

        public DateTime? DateImplement { get; set; }

        public int Id { get; set; }
        [Required]
        public int ClientId { get; private set; }
        public int? ImplementerId { get; private set; }

        public virtual Client Client { get; set; }
        public virtual Furniture Furniture { get; set; }
        public virtual Implementer? Implementer { get; set; }
        public static Order? Create(OrderBindingModel? model)
        {
            return new Order()
            {
                Id = model.Id,
                FurnitureId = model.FurnitureId,
                Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateImplement = model.DateImplement,
                ClientId = model.ClientId,
                ImplementerId = model.ImplementerId
            };
        }

        public void Update(OrderBindingModel? model)
        {
            if (model == null)
            {
                return;
            }
            Count = model.Count;
            Sum = model.Sum;
            Status = model.Status;
            DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            FurnitureId = FurnitureId,
            FurnitureName = Furniture.FurnitureName,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
            ClientId = ClientId,
            ClientFIO = Client.ClientFIO,
            ImplementerId = ImplementerId,
            ImplementerFIO = Implementer?.ImplementerFIO
        };

    }
}
