﻿using FurnitureAssemblyFileImplement.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace FurnitureAssemblyFileImplement
{
    public class DataFileSingleton
    {
         private static DataFileSingleton? instance;
        private readonly string WorkpieceFileName = "Workpiece.xml";
        private readonly string OrderFileName = "Order.xml";
        private readonly string FurnitureFileName = "Furniture.xml";
        private readonly string ClientFileName = "Client.xml";
        private readonly string ImplementerFileName = "Implementer.xml";
        private readonly string MessageFileName = "Message.xml";
        public List<Workpiece> Workpieces { get; private set; }
        public List<Order> Orders { get; private set; }
        public List<Furniture> Furnitures { get; private set; }
        public List<Client> Clients { get; private set; }
        public List<Implementer> Implementers { get; private set; }
        public List<MessageInfo> Messages { get; private set; }
        public static DataFileSingleton GetInstance()
        {
            if (instance == null)
            {
                instance = new DataFileSingleton();
            }
            return instance;
        }
        public void SaveWorkpieces() => SaveData(Workpieces, WorkpieceFileName, "Workpieces", x => x.GetXElement);
        public void SaveFurnitures() => SaveData(Furnitures, FurnitureFileName, "Furnitures", x => x.GetXElement);
        public void SaveOrders() => SaveData(Orders, OrderFileName, "Orders", x => x.GetXElement);
        public void SaveClients() => SaveData(Clients, ClientFileName, "Clients", x => x.GetXElement);
        public void SaveImplementers() => SaveData(Implementers, ImplementerFileName, "Implementers", x => x.GetXElement);
        public void SaveMessages() => SaveData(Messages, MessageFileName, "Messages", x => x.GetXElement);
        private DataFileSingleton()
        {
            Workpieces = LoadData(WorkpieceFileName, "Workpiece", x => Workpiece.Create(x)!)!;
            Furnitures = LoadData(FurnitureFileName, "Furniture", x => Furniture.Create(x)!)!;
            Orders = LoadData(OrderFileName, "Order", x => Order.Create(x)!)!;
            Clients = LoadData(ClientFileName, "Client", x => Client.Create(x)!)!;
            Implementers = LoadData(ImplementerFileName, "Implementer", x => Implementer.Create(x)!)!;
            Messages = LoadData(MessageFileName, "Messages", x => MessageInfo.Create(x)!)!;
        }
        private static List<T>? LoadData<T>(string filename, string xmlNodeName,
       Func<XElement, T> selectFunction)
        {
            if (File.Exists(filename))
            {
                return XDocument.Load(filename)?.Root?.Elements(xmlNodeName)?.Select(selectFunction)?.ToList();
            }
            return new List<T>();
        }
        private static void SaveData<T>(List<T> data, string filename, 
            string xmlNodeName, Func<T, XElement> selectFunction)
        {
            if (data != null)
            {
                new XDocument(new XElement(xmlNodeName,
               data.Select(selectFunction).ToArray())).Save(filename);
            }
        }
    }
}
