﻿using FurnitureAssemblyContracts.BindingModels;
using FurnitureAssemblyContracts.SearchModels;
using FurnitureAssemblyContracts.StoragesContracts;
using FurnitureAssemblyContracts.ViewModels;
using FurnitureAssemblyListImplement.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FurnitureAssemblyListImplement.Implements
{
    public class ImplementerStorage : IImplementerStorage
    {
        private readonly DataListSingleton _source;
        public ImplementerStorage()
        {
            _source = DataListSingleton.GetInstance();
        }
        public ImplementerViewModel? GetElement(ImplementerSearchModel model)
        {
            if (string.IsNullOrEmpty(model.ImplementerFIO) && !model.Id.HasValue && string.IsNullOrEmpty(model.Password))
            {
                return null;
            }

            foreach (var impl in _source.Implementers)
            {
                if (model.Id.HasValue && impl.Id == model.Id)
                {
                    return impl.GetViewModel;
                }

                if (model.ImplementerFIO != null && model.Password != null
                    && impl.ImplementerFIO.Equals(model.ImplementerFIO) && impl.Password.Equals(model.Password))
                {
                    return impl.GetViewModel;
                }

                if (model.ImplementerFIO != null && impl.ImplementerFIO.Equals(model.ImplementerFIO))
                {
                    return impl.GetViewModel;
                }
            }
            return null;
        }

        public List<ImplementerViewModel> GetFilteredList(ImplementerSearchModel model)
        {
            var result = new List<ImplementerViewModel>();
            if (string.IsNullOrEmpty(model.ImplementerFIO))
            {
                return result;
            }
            foreach (var impl in _source.Implementers)
            {
                if (impl.ImplementerFIO.Contains(model.ImplementerFIO))
                {
                    result.Add(impl.GetViewModel);
                }
            }
            return result;
        }

        public List<ImplementerViewModel> GetFullList()
        {
            var result = new List<ImplementerViewModel>();
            foreach (var component in _source.Implementers)
            {
                result.Add(component.GetViewModel);
            }
            return result;
        }

        public ImplementerViewModel? Insert(ImplementerBindingModel model)
        {
            model.Id = 1;
            foreach (var impl in _source.Implementers)
            {
                if (model.Id <= impl.Id)
                {
                    model.Id = impl.Id + 1;
                }
            }
            var newImpl = Implementer.Create(model);
            if (newImpl == null)
            {
                return null;
            }
            _source.Implementers.Add(newImpl);
            return newImpl.GetViewModel;
        }

        public ImplementerViewModel? Update(ImplementerBindingModel model)
        {
            foreach (var impl in _source.Implementers)
            {
                if (impl.Id == model.Id)
                {
                    impl.Update(model);
                    return impl.GetViewModel;
                }
            }
            return null;
        }

        public ImplementerViewModel? Delete(ImplementerBindingModel model)
        {
            for (int i = 0; i < _source.Implementers.Count; ++i)
            {
                if (_source.Implementers[i].Id == model.Id)
                {
                    var element = _source.Implementers[i];
                    _source.Implementers.RemoveAt(i);
                    return element.GetViewModel;
                }
            }
            return null;
        }
    }
}
